@@ -33,12 +33,14 @@ module Crypto.Classes
 	-- * Misc helper functions
 	, for
 	, (.::.)
+        , constTimeEq
 	) where
 
 import Data.Serialize
 import qualified Data.ByteString.Lazy as L
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Internal as I
+import Data.Bits ((.|.), xor)
 import Data.List (foldl')
 import Data.Word (Word64)
 import Data.Tagged
@@ -114,14 +116,6 @@ makeBlocks msg len = go (L.toChunks msg)
 		[] -> ([], x)
 		(a:as) -> go (B.append x a : as)
 
--- |Obtain a tagged value for a given type
-for :: Tagged a b -> a -> b
-for t _ = unTagged t
-
--- |Infix `for` operator
-(.::.) :: Tagged a b -> a -> b
-(.::.) = for
-
 -- |The BlockCipher class is intended as the generic interface
 -- targeted by maintainers of Haskell cipher implementations.
 -- Using this generic interface higher level functions
@@ -211,3 +205,28 @@ buildSigningKeyPairIO bl = do
 	case buildSigningPair g bl of
 		Left err -> return $ Left err
 		Right (k,_) -> return $ Right k
+
+-- |Obtain a tagged value for a given type
+for :: Tagged a b -> a -> b
+for t _ = unTagged t
+
+-- |Infix `for` operator
+(.::.) :: Tagged a b -> a -> b
+(.::.) = for
+
+-- | Checks two bytestrings for equality without breaches for
+-- timing attacks.
+--
+-- Semantically, @constTimeEq = (==)@.  However, @x == y@ takes less
+-- time when the first byte is different than when the first byte
+-- is equal.  This side channel allows an attacker to mount a
+-- timing attack.  On the other hand, @constTimeEq@ always takes the
+-- same time regardless of the bytestrings' contents.
+--
+-- You should always use @constTimeEq@ when comparing hashes,
+-- otherwise you may leave a significant security hole
+-- (cf. <http://codahale.com/a-lesson-in-timing-attacks/>).
+constTimeEq :: B.ByteString -> B.ByteString -> Bool
+constTimeEq s1 s2 =
+  B.length s1 == B.length s2 &&
+  foldl' (.|.) 0 (B.zipWith xor s1 s2) == 0
